@@ -325,7 +325,7 @@ ENTER(0x160);
        "%ld "
        "%lu %"KLF"u %"KLF"u %"KLF"u %"KLF"u %"KLF"u "
        "%*s %*s %*s %*s " /* discard, no RT signals & Linux 2.1 used hex */
-       "%"KLF"u %lu %lu "
+       "%"KLF"u %*lu %*lu "
        "%d %d "
        "%lu %lu",
        &P->state,
@@ -334,13 +334,13 @@ ENTER(0x160);
        &P->utime, &P->stime, &P->cutime, &P->cstime,
        &P->priority, &P->nice,
        &P->nlwp,
-       &P->it_real_value,
+       &P->alarm,
        &P->start_time,
        &P->vsize,
        &P->rss,
        &P->rss_rlim, &P->start_code, &P->end_code, &P->start_stack, &P->kstk_esp, &P->kstk_eip,
 /*     P->signal, P->blocked, P->sigignore, P->sigcatch,   */ /* can't use */
-       &P->wchan, &P->nswap, &P->cnswap,
+       &P->wchan, /* &P->nswap, &P->cnswap, */  /* nswap and cnswap dead for 2.4.xx and up */
 /* -- Linux 2.0.35 ends here -- */
        &P->exit_signal, &P->processor,  /* 2.2.1 ends with "exit_signal" */
 /* -- Linux 2.2.8 to 2.5.17 end here -- */
@@ -521,23 +521,28 @@ static proc_t* simple_readproc(PROCTAB *restrict const PT, proc_t *restrict cons
        }
     }
 
-    /* some number->text resolving which is time consuming */
+    // if multithreaded, some values are crap
+    if(p->nlwp > 1){
+      p->wchan = (KLONG)~0ull;
+    }
+
+    /* some number->text resolving which is time consuming and kind of insane */
     if (flags & PROC_FILLUSR){
-	strncpy(p->euser,   user_from_uid(p->euid), sizeof p->euser);
+	memcpy(p->euser,   user_from_uid(p->euid), sizeof p->euser);
         if(flags & PROC_FILLSTATUS) {
-            strncpy(p->ruser,   user_from_uid(p->ruid), sizeof p->ruser);
-            strncpy(p->suser,   user_from_uid(p->suid), sizeof p->suser);
-            strncpy(p->fuser,   user_from_uid(p->fuid), sizeof p->fuser);
+            memcpy(p->ruser,   user_from_uid(p->ruid), sizeof p->ruser);
+            memcpy(p->suser,   user_from_uid(p->suid), sizeof p->suser);
+            memcpy(p->fuser,   user_from_uid(p->fuid), sizeof p->fuser);
         }
     }
 
-    /* some number->text resolving which is time consuming */
+    /* some number->text resolving which is time consuming and kind of insane */
     if (flags & PROC_FILLGRP){
-        strncpy(p->egroup, group_from_gid(p->egid), sizeof p->egroup);
+        memcpy(p->egroup, group_from_gid(p->egid), sizeof p->egroup);
         if(flags & PROC_FILLSTATUS) {
-            strncpy(p->rgroup, group_from_gid(p->rgid), sizeof p->rgroup);
-            strncpy(p->sgroup, group_from_gid(p->sgid), sizeof p->sgroup);
-            strncpy(p->fgroup, group_from_gid(p->fgid), sizeof p->fgroup);
+            memcpy(p->rgroup, group_from_gid(p->rgid), sizeof p->rgroup);
+            memcpy(p->sgroup, group_from_gid(p->sgid), sizeof p->sgroup);
+            memcpy(p->fgroup, group_from_gid(p->fgid), sizeof p->fgroup);
         }
     }
 
@@ -606,21 +611,21 @@ static proc_t* simple_readtask(PROCTAB *restrict const PT, const proc_t *restric
 
     /* some number->text resolving which is time consuming */
     if (flags & PROC_FILLUSR){
-	strncpy(t->euser,   user_from_uid(t->euid), sizeof t->euser);
+	memcpy(t->euser,   user_from_uid(t->euid), sizeof t->euser);
         if(flags & PROC_FILLSTATUS) {
-            strncpy(t->ruser,   user_from_uid(t->ruid), sizeof t->ruser);
-            strncpy(t->suser,   user_from_uid(t->suid), sizeof t->suser);
-            strncpy(t->fuser,   user_from_uid(t->fuid), sizeof t->fuser);
+            memcpy(t->ruser,   user_from_uid(t->ruid), sizeof t->ruser);
+            memcpy(t->suser,   user_from_uid(t->suid), sizeof t->suser);
+            memcpy(t->fuser,   user_from_uid(t->fuid), sizeof t->fuser);
         }
     }
 
     /* some number->text resolving which is time consuming */
     if (flags & PROC_FILLGRP){
-        strncpy(t->egroup, group_from_gid(t->egid), sizeof t->egroup);
+        memcpy(t->egroup, group_from_gid(t->egid), sizeof t->egroup);
         if(flags & PROC_FILLSTATUS) {
-            strncpy(t->rgroup, group_from_gid(t->rgid), sizeof t->rgroup);
-            strncpy(t->sgroup, group_from_gid(t->sgid), sizeof t->sgroup);
-            strncpy(t->fgroup, group_from_gid(t->fgid), sizeof t->fgroup);
+            memcpy(t->rgroup, group_from_gid(t->rgid), sizeof t->rgroup);
+            memcpy(t->sgroup, group_from_gid(t->sgid), sizeof t->sgroup);
+            memcpy(t->fgroup, group_from_gid(t->fgid), sizeof t->fgroup);
         }
     }
 
