@@ -11,18 +11,19 @@
 
 
 #include "procps.h"
+#include "pwcache.h"
 
 #define SIGNAL_STRING
 
 EXTERN_C_BEGIN
 
-// ld	cutime, cstime, priority, nice, timeout, it_real_value, rss,
+// ld	cutime, cstime, priority, nice, timeout, alarm, rss,
 // c	state,
 // d	ppid, pgrp, session, tty, tpgid,
 // s	signal, blocked, sigignore, sigcatch,
 // lu	flags, min_flt, cmin_flt, maj_flt, cmaj_flt, utime, stime,
 // lu	rss_rlim, start_code, end_code, start_stack, kstk_esp, kstk_eip,
-// lu	start_time, vsize, wchan, nswap, cnswap,
+// lu	start_time, vsize, wchan,
 
 // This is to help document a transition from pid to tgid/tid caused
 // by the introduction of thread support. It is used in cases where
@@ -73,7 +74,7 @@ typedef struct proc_t {
 	priority,	// stat            kernel scheduling priority
 	nice,		// stat            standard unix nice level of process
 	rss,		// stat            resident set size from /proc/#/stat (pages)
-	it_real_value,	// stat            ?
+	alarm,		// stat            ?
     // the next 7 members come from /proc/#/statm
 	size,		// statm           total # of pages of memory
 	resident,	// statm           number of resident set (non-swapped) pages (4k)
@@ -98,9 +99,7 @@ typedef struct proc_t {
 	min_flt,	// stat            number of minor page faults since process start
 	maj_flt,	// stat            number of major page faults since process start
 	cmin_flt,	// stat            cumulative min_flt of process and child processes
-	cmaj_flt,	// stat            cumulative maj_flt of process and child processes
-	nswap,		// stat            ?
-	cnswap;		// stat            cumulative nswap ?
+	cmaj_flt;	// stat            cumulative maj_flt of process and child processes
     unsigned KLONG
 	start_code,	// stat            address of beginning of code segment
 	end_code,	// stat            address of end of code segment
@@ -113,14 +112,14 @@ typedef struct proc_t {
 	**cmdline;	// (special)       command line string vector (/proc/#/cmdline)
     char
 	// Be compatible: Digital allows 16 and NT allows 14 ???
-    	euser[16],	// stat(),status   effective user name
-    	ruser[16],	// status          real user name
-    	suser[16],	// status          saved user name
-    	fuser[16],	// status          filesystem user name
-    	rgroup[16],	// status          real group name
-    	egroup[16],	// status          effective group name
-    	sgroup[16],	// status          saved group name
-    	fgroup[16],	// status          filesystem group name
+    	euser[P_G_SZ],	// stat(),status   effective user name
+    	ruser[P_G_SZ],	// status          real user name
+    	suser[P_G_SZ],	// status          saved user name
+    	fuser[P_G_SZ],	// status          filesystem user name
+    	rgroup[P_G_SZ],	// status          real group name
+    	egroup[P_G_SZ],	// status          effective group name
+    	sgroup[P_G_SZ],	// status          saved group name
+    	fgroup[P_G_SZ],	// status          filesystem group name
     	cmd[16];	// stat,status     basename of executable file in call to exec(2)
     struct proc_t
 	*ring,		// n/a             thread group ring
