@@ -191,16 +191,20 @@ static unsigned long find_elf_note(unsigned long findme){
 
 static void init_libproc(void) __attribute__((constructor));
 static void init_libproc(void){
-  /* ought to count CPUs in /proc/stat instead of relying
-   * on glibc, which foolishly tries to parse /proc/cpuinfo
-   */
-  smp_num_cpus = sysconf(_SC_NPROCESSORS_CONF); // or _SC_NPROCESSORS_ONLN
+  // ought to count CPUs in /proc/stat instead of relying
+  // on glibc, which foolishly tries to parse /proc/cpuinfo
+  //
+  // SourceForge has an old Alpha running Linux 2.2.20 that
+  // appears to have a non-SMP kernel on a 2-way SMP box.
+  // _SC_NPROCESSORS_CONF returns 2, resulting in HZ=512
+  // _SC_NPROCESSORS_ONLN returns 1, which should work OK
+  smp_num_cpus = sysconf(_SC_NPROCESSORS_ONLN);
   if(smp_num_cpus<1) smp_num_cpus=1; /* SPARC glibc is buggy */
 
   if(linux_version_code > LINUX_VERSION(2, 4, 0)){ 
     Hertz = find_elf_note(AT_CLKTCK);
     if(Hertz!=NOTE_NOT_FOUND) return;
-    fprintf(stderr, "2.4 kernel w/o ELF notes? -- report to albert@users.sf.net\n");
+    fprintf(stderr, "2.4 kernel w/o ELF notes? -- report this\n");
   }
   old_Hertz_hack();
 }
@@ -307,26 +311,24 @@ static void getrunners(unsigned int *restrict running, unsigned int *restrict bl
   if((proc=opendir("/proc"))==NULL) crash("/proc");
 
   while(( ent=readdir(proc) )) {
-    unsigned size;
+    char tbuf[32];
+    char *cp;
     int fd;
-    char filename[80];
     char c;
+
     if (!isdigit(ent->d_name[0])) continue;
-    sprintf(filename, "/proc/%s/stat", ent->d_name);
-    fd = open(filename, O_RDONLY, 0);
+    sprintf(tbuf, "/proc/%s/stat", ent->d_name);
+
+    fd = open(tbuf, O_RDONLY, 0);
     if (fd == -1) continue;
-    read(fd,buff,BUFFSIZE-1);
-    sscanf(
-      buff,
-      "%*d %*s %c "
-      "%*d %*d %*d %*d %*d %*u %*u"
-      " %*u %*u %*u %*d %*d %*d %*d %*d %*d %*u %*u %*d %*u %u"
-      /*  " %*u %*u %*u %*u %*u %*u %*u %*u %*u %*u %*u\n"  */ ,
-      &c,
-      &size
-    );
+    memset(tbuf, '\0', sizeof tbuf); // didn't feel like checking read()
+    read(fd, tbuf, sizeof tbuf - 1); // need 32 byte buffer at most
     close(fd);
 
+    cp = strrchr(tbuf, ')');
+    if(!cp) continue;
+    c = cp[2];
+
     if (c=='R') {
       (*running)++;
       continue;
@@ -347,6 +349,7 @@ void getstat(jiff *restrict cuse, jiff *restrict cice, jiff *restrict csys, jiff
 	     unsigned int *restrict running, unsigned int *restrict blocked,
 	     unsigned int *restrict btime, unsigned int *restrict processes) {
   static int fd;
+  unsigned long long llbuf = 0;
   int need_vmstat_file = 0;
   int need_proc_scan = 0;
   const char* b;
@@ -376,10 +379,12 @@ void getstat(jiff *restrict cuse, jiff *restrict cice, jiff *restrict csys, jiff
   else need_vmstat_file = 1;
 
   b = strstr(buff, "intr ");
-  if(b) sscanf(b,  "intr %u", intr);
+  if(b) sscanf(b,  "intr %Lu", &llbuf);
+  *intr = llbuf;
 
   b = strstr(buff, "ctxt ");
-  if(b) sscanf(b,  "ctxt %u", ctxt);
+  if(b) sscanf(b,  "ctxt %Lu", &llbuf);
+  *ctxt = llbuf;
 
   b = strstr(buff, "btime ");
   if(b) sscanf(b,  "btime %u", btime);
@@ -674,20 +679,6 @@ nextline:
 ///////////////////////////////////////////////////////////////////////
 // based on Fabian Frederick's /proc/diskstats parser
 
-static unsigned int getFileLines(const char* szFile){
-  char szBuffer[1024];
-  FILE *fdiskStat;
-  int lines=0;
-  if ((fdiskStat=fopen (szFile,"rb"))){
-    while (fgets(szBuffer, 1024, fdiskStat)){
-      lines++;
-    }
-    fclose(fdiskStat);
-  } 
-  return lines;
-}
-
-/////////////////////////////////////////////////////////////////////////////
 
 unsigned int getpartitions_num(struct disk_stat *disks, int ndisks){
   int i=0;
@@ -704,31 +695,28 @@ unsigned int getpartitions_num(struct disk_stat *disks, int ndisks){
 
 unsigned int getdiskstat(struct disk_stat **disks, struct partition_stat **partitions){
   FILE* fd;
-  int units,
-      i,
-      disk_type,
-      disk_num,
-      cDisk=0,
-      cPartition=0;
+  int cDisk = 0;
+  int cPartition = 0;
+  int fields;
+  unsigned dummy;
 
   *disks = NULL;
   *partitions = NULL;
   buff[BUFFSIZE-1] = 0; 
-  units = getFileLines("/proc/diskstats");
   fd = fopen("/proc/diskstats", "rb");
   if(!fd) crash("/proc/diskstats");
 
-  for (i=0; i<units; i++){
+  for (;;) {
     if (!fgets(buff,BUFFSIZE-1,fd)){
       fclose(fd);
-      crash("/proc/diskstats");
+      break;
     }
-    sscanf(buff, "    %d    %d", &disk_type, &disk_num);
-    if (disk_num == 0){
+    fields = sscanf(buff, " %*d %*d %*s %*u %*u %*u %*u %*u %*u %*u %*u %*u %*u %u", &dummy);
+    if (fields == 1){
       (*disks) = realloc(*disks, (cDisk+1)*sizeof(struct disk_stat));
-      sscanf(buff,  "   %d    %*d %15s %u %u %llu %u %u %u %llu %u %u %u %u",
-        &(*disks)[cDisk].disk_type,
-        //&unused,
+      sscanf(buff,  "   %*d    %*d %15s %u %u %llu %u %u %u %llu %u %u %u %u",
+        //&disk_major,
+        //&disk_minor,
         (*disks)[cDisk].disk_name,
         &(*disks)[cDisk].reads,
         &(*disks)[cDisk].merged_reads,
@@ -747,20 +735,20 @@ unsigned int getdiskstat(struct disk_stat **disks, struct partition_stat **parti
     }else{
       (*partitions) = realloc(*partitions, (cPartition+1)*sizeof(struct partition_stat));
       fflush(stdout);
-      sscanf(buff,  "   %d    %d %15s %u %llu %u %u",
-        &(*partitions)[cPartition].disk_type,
-        &(*partitions)[cPartition].partition_num,
+      sscanf(buff,  "   %*d    %*d %15s %u %llu %u %u",
+        //&part_major,
+        //&part_minor,
         (*partitions)[cPartition].partition_name,
         &(*partitions)[cPartition].reads,
         &(*partitions)[cPartition].reads_sectors,
         &(*partitions)[cPartition].writes,
         &(*partitions)[cPartition].requested_writes
       );
-      (*partitions)[cPartition++].parent_disk = &((*disks)[cDisk-1]);
+      (*partitions)[cPartition++].parent_disk = cDisk-1;
       (*disks)[cDisk-1].partitions++;	
     }
   }
-  fclose(fd);
+
   return cDisk;
 }
 
