@@ -171,6 +171,7 @@ static const char *set_personality(void){
   {"sgi",      &&case_sgi},
   {"solaris2", &&case_solaris2},
   {"sunos4",   &&case_sunos4},
+  {"svr4",     &&case_svr4},
   {"sysv",     &&case_sysv},
   {"tru64",    &&case_tru64},
   {"unix",     &&case_unix},
@@ -230,10 +231,10 @@ static const char *set_personality(void){
 
   case_debian:  /* Toss this? They don't seem to care much. */
   case_gnu:
-    personality = PER_GOOD_o | PER_CUMUL_MARKED | PER_OLD_m;
+    personality = PER_GOOD_o | PER_OLD_m;
     prefer_bsd_defaults = 1;
     sysv_f_format  = "RD_f";
-    /* sysv_fl_format = "RD_fl"; */   /* Debian can't do this! */
+    /* sysv_fl_format = "RD_fl"; */   /* old Debian ps can't do this! */
     sysv_j_format  = "RD_j";
     sysv_l_format  = "RD_l";
     return NULL;
@@ -304,10 +305,17 @@ static const char *set_personality(void){
 
   case_hp:
   case_hpux:
-  case_posix:
-  case_sco:
-  case_solaris2:
+    personality = PER_BROKEN_o | PER_HPUX_x;
+    return NULL;
+
+  case_svr4:
   case_sysv:
+  case_sco:
+    personality = PER_BROKEN_o | PER_SVR4_x;
+    return NULL;
+
+  case_posix:
+  case_solaris2:
   case_unix95:
   case_unix98:
   case_unix:
