@@ -12,9 +12,6 @@
 /* Ought to have debug print stuff like this:
  * #define Print(fmt, args...) printf("Debug: " fmt, ## args)
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -256,6 +253,16 @@ static const char *parse_sysv_option(void){
       trace("-H Process hierarchy (like ASCII art forest option)\n");
       forest_type = 'u';
       break;
+#if 0
+    case 'J':  // specify list of job IDs in hex (IRIX) -- like HP "-R" maybe?
+      trace("-J select by job ID\n");  // want a JID ("jid") for "-j" too
+      arg=get_opt_arg();
+      if(!arg) return "List of jobs must follow -J.";
+      err=parse_list(arg, parse_jid);
+      if(err) return err;
+      selection_list->typecode = SEL_JID;
+      return NULL; /* can't have any more options */
+#endif
     case 'L':  /*  */
       /* In spite of the insane 2-level thread system, Sun appears to
        * have made this option Linux-compatible. If a process has N
@@ -267,7 +274,7 @@ static const char *parse_sysv_option(void){
       thread_flags |= TF_U_L;
 //      format_modifiers |= FM_L;
       break;
-    case 'M':  /* someday, maybe, we will have MAC like SGI's Irix */
+    case 'M':  // typically the SE Linux context
       trace("-M Print security label for Mandatory Access Control.\n");
       format_modifiers |= FM_M;
       break;
@@ -282,14 +289,18 @@ static const char *parse_sysv_option(void){
       defer_sf_option(arg, SF_U_O);
       return NULL; /* can't have any more options */
     case 'P':     /* SunOS 5 "psr" or unknown HP/UX feature */
-      trace("-P adds columns of PRM info (HP) or PSR column (Sun)\n");
+      trace("-P adds columns of PRM info (HP-UX), PSR (SunOS), or capabilities (IRIX)\n");
       format_modifiers |= FM_P;
       break;
-#ifdef WE_UNDERSTAND_THIS
-    case 'R':     /* unknown HP/UX feature */
-      trace("-R selects PRM groups\n");
-      return "Don't understand PRM on Linux.";
-      break;
+#if 0
+    case 'R':    // unknown HP/UX feature, like IRIX "-J" maybe?
+      trace("-R select by PRM group\n");
+      arg=get_opt_arg();
+      if(!arg) return "List of PRM groups must follow -R.";
+      err=parse_list(arg, parse_prm);
+      if(err) return err;
+      selection_list->typecode = SEL_PRM;
+      return NULL; /* can't have any more options */
 #endif
     case 'T':
       /* IRIX 6.5 docs suggest POSIX threads get shown individually.
@@ -314,12 +325,12 @@ static const char *parse_sysv_option(void){
       exclusive("-V");
       display_version();
       exit(0);
-#if 0
+    // This must be verified against SVR4-MP. (UnixWare or Powermax)
+    // Leave it undocumented until that problem is solved.
     case 'Z':     /* full Mandatory Access Control level info */
       trace("-Z shows full MAC info\n");
-      return "Don't understand MAC on Linux.";
+      format_modifiers |= FM_M;
       break;
-#endif
     case 'a':
       trace("-a select all with a tty, but omit session leaders.\n");
       simple_select |= SS_U_a;
@@ -397,7 +408,7 @@ static const char *parse_sysv_option(void){
       if(err) return err;
       selection_list->typecode = SEL_PID;
       return NULL; /* can't have any more options */
-#ifdef KNOW_WHAT_TO_DO_WITH_THIS
+#if 0
     case 'r':
       trace("-r some Digital Unix thing about warnings...\n");
       trace("   or SCO's option to chroot() for new /proc and /dev.\n");
@@ -432,20 +443,29 @@ static const char *parse_sysv_option(void){
       trace("-w wide output.\n");
       w_count++;
       break;
-#ifdef NOBODY_HAS_BSD_HABITS_ANYMORE
-    case 'x':     /* Same as -y, but for System V Release 4 MP */
-      trace("-x works like Sun Solaris & SCO Unixware -y option\n");
-      format_modifiers |= FM_x;
-      break;
-#endif
+    case 'x':  /* behind personality until "ps -ax" habit is uncommon */
+      if(personality & PER_SVR4_x){
+        // Same as -y, but for System V Release 4 MP
+        trace("-x works like Sun Solaris & SCO Unixware -y option\n");
+        format_modifiers |= FM_y;
+        break;
+      }
+      if(personality & PER_HPUX_x){
+        trace("-x extends the command line\n");
+        w_count += 2;
+        unix_f_option = 1;
+        break;
+      }
+      return "Must set personality to get -x option.";
     case 'y':  /* Sun's -l hack (also: Irix "lnode" resource control info) */
       trace("-y Print lnone info in UID/USER column or do Sun -l hack.\n");
       format_modifiers |= FM_y;
       break;
 #if 0
+    // This must be verified against SVR4-MP (UnixWare or Powermax)
     case 'z':     /* alias of Mandatory Access Control level info */
       trace("-z shows aliased MAC info\n");
-      return "Don't understand MAC on Linux.";
+      format_modifiers |= FM_M;
       break;
 #endif
     case '-':
@@ -509,12 +529,10 @@ static const char *parse_bsd_option(void){
       exclusive("L");
       print_format_specifiers();
       exit(0);
-#if 0
-    case 'M':
-      trace("M junk (use alternate core)\n");
-      return "Option M is unsupported, try N or -n instead.";
+    case 'M':   // undocumented for now: these are proliferating!
+      trace("M MacOS X thread display, like AIX/Tru64\n");
+      thread_flags |= TF_B_m;
       break;
-#endif
     case 'N': /* end */
       trace("N Specify namelist file\n");
       arg=get_opt_arg();
@@ -601,11 +619,16 @@ static const char *parse_bsd_option(void){
       trace("j job control format\n");
       format_flags |= FF_Bj;
       break;
-#if 0
-    case 'k':    // OpenBSD: don't hide "kernel threads" -- like the swapper?
-      trace("k Print LWP (thread) info.\n");   // was: Use /vmcore as c-dumpfile\n");
-      break;
-#endif
+    case 'k':
+      // OpenBSD: don't hide "kernel threads" -- like the swapper?
+      // trace("k Print LWP (thread) info.\n");   // was: Use /vmcore as c-dumpfile\n");
+
+      // NetBSD, and soon (?) FreeBSD: sort-by-keyword
+      trace("k Specify sorting keywords.\n");
+      arg=get_opt_arg();
+      if(!arg) return "Long sort specification must follow 'k'.";
+      defer_sf_option(arg, SF_G_sort);
+      return NULL; /* can't have any more options */
     case 'l':
       trace("l Display long format\n");
       format_flags |= FF_Bl;
@@ -686,6 +709,17 @@ static const char *parse_bsd_option(void){
       trace("x Select processes without controlling ttys\n");
       simple_select |= SS_B_x;
       break;
+#if 0
+    case 'y':
+      // DragonFlyBSD iac (interactivity measure) format
+      // uid,pid,ppid,cpu,pri,iac,nice,wchan,state,tt,time,command
+      // (they use 'Y' to sort by this "iac" thing; 'y' implies 'Y')
+      // Range is -127 .. 127, with lower numbers being more
+      // interactive and higher numbers more batch-like.
+      trace("y Display interactivity measure\n");
+      format_flags |= FF_Bv;
+      break;
+#endif
     case '-':
       return "Embedded '-' among BSD options makes no sense.";
       break;
@@ -876,7 +910,7 @@ static const char *parse_gnu_option(void){
   case_help:
     trace("--help\n");
     exclusive("--help");
-    fwrite(help_message,1,strlen(help_message),stderr);
+    fwrite(help_message,1,strlen(help_message),stdout);
     exit(0);
     return NULL;
   case_info:
@@ -1178,8 +1212,13 @@ try_bsd:
   // get seriously confused. Ask yourself if users would freak out
   // about "ps -aux" suddenly changing behavior if a user "x" were
   // added to the system.
+  //
+  // Also, a "-x" option is coming. It's already there in fact,
+  // for some non-default personalities. So "ps -ax" will parse
+  // as SysV options... and you're screwed if you've been patching
+  // out the friendly warning. Cut-over is likely to be in 2005.
   if(!(personality & PER_FORCE_BSD))
-    fprintf(stderr, "Warning: bad syntax, perhaps a bogus '-'? See http://procps.sf.net/faq.html\n");
+    fprintf(stderr, "Warning: bad ps syntax, perhaps a bogus '-'? See http://procps.sf.net/faq.html\n");
   // Remember: contact albert@users.sf.net or procps-feedback@lists.sf.net
   // if you should feel tempted. Be damn sure you understand all
   // the issues. The same goes for other stuff too, BTW. Please ask.
