@@ -15,6 +15,8 @@
  * Changelog:
  *            v1.01:
  *                   - added -p <preload> to preload values from a file
+ *            Horms: 
+ *                   - added -q to be quiet when modifying values
  *
  * Changes by Albert Cahalan, 2002.
  */
@@ -43,9 +45,11 @@ static bool false = 0;
 
 static const char PROC_PATH[] = "/proc/sys/";
 static const char DEFAULT_PRELOAD[] = "/etc/sysctl.conf";
+static bool NameOnly;
 static bool PrintName;
 static bool PrintNewline;
 static bool IgnoreError;
+static bool Quiet;
 
 /* error messages */
 static const char ERR_UNKNOWN_PARAMETER[] = "error: Unknown parameter '%s'\n";
@@ -80,9 +84,9 @@ static void slashdot(char *restrict p, char old, char new){
  */
 static int Usage(const char *restrict const name) {
    printf("usage:  %s [-n] [-e] variable ... \n"
-          "        %s [-n] [-e] -w variable=value ... \n" 
+          "        %s [-n] [-e] [-q] -w variable=value ... \n" 
           "        %s [-n] [-e] -a \n" 
-          "        %s [-n] [-e] -p <file>   (default /etc/sysctl.conf) \n"
+          "        %s [-n] [-e] [-q] -p <file>   (default /etc/sysctl.conf) \n"
           "        %s [-n] [-e] -A\n", name, name, name, name, name);
    return -1;
 }
@@ -112,7 +116,7 @@ static char *StripLeadingAndTrailingSpaces(char *oneline) {
    return t;
 }
 
-
+static int DisplayAll(const char *restrict const path);
 
 /*
  *     Read a sysctl setting 
@@ -160,16 +164,43 @@ static int ReadSetting(const char *restrict const name) {
          break;
       }
    } else {
-      while(fgets(inbuf, 1024, fp)) {
-         /* already has the \n in it */
-         if (PrintName) {
-            fprintf(stdout, "%s = %s", outname, inbuf);
-         } else {
-            if (!PrintNewline) {
-              char *nlptr = strchr(inbuf,'\n');
-              if(nlptr) *nlptr='\0';
+      if(fgets(inbuf, sizeof inbuf - 1, fp)) {
+         // this loop is required, see
+         // /sbin/sysctl -a | egrep -6 dev.cdrom.info
+         do {
+            if (NameOnly) {
+               fprintf(stdout, "%s\n", outname);
+            } else {
+               /* already has the \n in it */
+               if (PrintName) {
+                  fprintf(stdout, "%s = %s", outname, inbuf);
+               } else {
+                  if (!PrintNewline) {
+                    char *nlptr = strchr(inbuf,'\n');
+                    if(nlptr) *nlptr='\0';
+                  }
+                  fprintf(stdout, "%s", inbuf);
+               }
             }
-            fprintf(stdout, "%s", inbuf);
+         } while(fgets(inbuf, sizeof inbuf - 1, fp));
+      } else {
+         switch(errno) {
+         case EACCES:
+            fprintf(stderr, ERR_PERMISSION_DENIED, outname);
+            rc = -1;
+            break;
+         case EISDIR:{
+            size_t len;
+            len = strlen(tmpname);
+            tmpname[len] = '/';
+            tmpname[len+1] = '\0';
+            rc = DisplayAll(tmpname);
+            break;
+         }
+         default:
+            fprintf(stderr, ERR_UNKNOWN_READING, errno, outname);
+            rc = -1;
+            break;
          }
       }
       fclose(fp);
@@ -186,12 +217,11 @@ static int ReadSetting(const char *restrict const name) {
  *     Display all the sysctl settings 
  *
  */
-static int DisplayAll(const char *restrict const path, bool ShowTableUtil) {
+static int DisplayAll(const char *restrict const path) {
    int rc = 0;
    int rc2;
    DIR *restrict dp;
    struct dirent *restrict de;
-   char *restrict tmpdir;
    struct stat ts;
 
    dp = opendir(path);
@@ -200,17 +230,19 @@ static int DisplayAll(const char *restrict const path, bool ShowTableUtil) {
       fprintf(stderr, ERR_OPENING_DIR, path);
       rc = -1;
    } else {
-      readdir(dp); readdir(dp);   /* skip . and .. */
+      readdir(dp);  // skip .
+      readdir(dp);  // skip ..
       while (( de = readdir(dp) )) {
+         char *restrict tmpdir;
          tmpdir = (char *restrict)malloc(strlen(path)+strlen(de->d_name)+2);
          sprintf(tmpdir, "%s%s", path, de->d_name);
-         rc2 = stat(tmpdir, &ts);       /* should check this return code */
+         rc2 = stat(tmpdir, &ts);
          if (rc2 != 0) {
             perror(tmpdir);
          } else {
             if (S_ISDIR(ts.st_mode)) {
                strcat(tmpdir, "/");
-               DisplayAll(tmpdir, ShowTableUtil);
+               DisplayAll(tmpdir);
             } else {
                rc |= ReadSetting(tmpdir+strlen(PROC_PATH));
             }
@@ -287,16 +319,28 @@ static int WriteSetting(const char *setting) {
          break;
       }
    } else {
-      fprintf(fp, "%s\n", value);
-      fclose(fp);
-
-      if (PrintName) {
-         fprintf(stdout, "%s = %s\n", outname, value);
+      rc = fprintf(fp, "%s\n", value);
+      if (rc < 0) {
+         fprintf(stderr, ERR_UNKNOWN_WRITING, errno, outname);
+         fclose(fp);
       } else {
-         if (PrintNewline)
-            fprintf(stdout, "%s\n", value);
-         else
-            fprintf(stdout, "%s", value);
+         rc=fclose(fp);
+         if (rc != 0) 
+            fprintf(stderr, ERR_UNKNOWN_WRITING, errno, outname);
+      }
+      if (rc==0 && !Quiet) {
+         if (NameOnly) {
+            fprintf(stdout, "%s\n", outname);
+         } else {
+            if (PrintName) {
+               fprintf(stdout, "%s = %s\n", outname, value);
+            } else {
+               if (PrintNewline)
+                  fprintf(stdout, "%s\n", value);
+               else
+                  fprintf(stdout, "%s", value);
+            }
+         }
       }
    }
 
@@ -353,6 +397,7 @@ static int Preload(const char *restrict const filename) {
       while ((*value == ' ' || *value == '\t') && *value != 0)
          value++;
 
+      // should NameOnly affect this?
       sprintf(buffer, "%s=%s", name, value);
       rc |= WriteSetting(buffer);
    }
@@ -367,7 +412,7 @@ static int Preload(const char *restrict const filename) {
  *    Main... 
  *
  */
-int main(int argc, char **argv) {
+int main(int argc, char *argv[]) {
    const char *me = (const char *)basename(argv[0]);
    bool SwitchesAllowed = true;
    bool WriteMode = false;
@@ -377,6 +422,7 @@ int main(int argc, char **argv) {
    PrintName = true;
    PrintNewline = true;
    IgnoreError = false;
+   Quiet = false;
 
    if (argc < 2) {
        return Usage(me);
@@ -407,27 +453,41 @@ int main(int argc, char **argv) {
               PrintName = false;
            break;
          case 'e':
+              // For FreeBSD, -e means a "%s=%s\n" format. ("%s: %s\n" default)
+              // We (and NetBSD) use "%s = %s\n" always, and -e to ignore errors.
               IgnoreError = true;
            break;
+         case 'N':
+              NameOnly = true;
+           break;
          case 'w':
               SwitchesAllowed = false;
               WriteMode = true;
            break;
+         case 'f':  // the NetBSD way
          case 'p':
               argv++;
               if (argv && *argv && **argv) {
                  preloadfile = *argv;
               }
               return Preload(preloadfile);
-         case 'a': /* string and integer values (for Linux, all of them) */
-         case 'A': /* the above, including "opaques" (would be unprintable) */
-         case 'X': /* the above, with opaques completly printed in hex */
+	 case 'q':
+	      Quiet = true;
+	   break;
+	 case 'o':  // BSD: binary values too, 1st 16 bytes in hex
+	 case 'x':  // BSD: binary values too, whole thing in hex
+	      /* does nothing */ ;
+	   break;
+         case 'a': // string and integer values (for Linux, all of them)
+         case 'A': // same as -a -o
+         case 'X': // same as -a -x
               SwitchesAllowed = false;
-              return DisplayAll(PROC_PATH, ((*argv)[1] == 'a') ? false : true);
+              return DisplayAll(PROC_PATH);
          case 'V':
               fprintf(stdout, "sysctl (%s)\n",procps_version);
               exit(0);
-         case 'h':
+         case 'd':   // BSD: print description ("vm.kvm_size: Size of KVM")
+         case 'h':   // BSD: human-readable (did FreeBSD 5 make -e default?)
          case '?':
               return Usage(me);
          default:
@@ -435,8 +495,10 @@ int main(int argc, char **argv) {
               return Usage(me);
          }
       } else {
+         if (NameOnly && Quiet)   // nonsense
+            return Usage(me);
          SwitchesAllowed = false;
-         if (WriteMode)
+         if (WriteMode || index(*argv, '='))
             ReturnCode = WriteSetting(*argv);
          else
             ReturnCode = ReadSetting(*argv);
