@@ -1,5 +1,4 @@
-/*
- * watch - execute a program repeatedly, displaying the output fullscreen
+/* watch -- execute a program repeatedly, displaying output fullscreen
  *
  * Based on the original 1991 'watch' by Tony Rems <rembo@unisoft.com>
  * (with mods and corrections by Francois Pinard).
@@ -7,7 +6,11 @@
  * Substantially reworked, new features (differences option, SIGWINCH
  * handling, unlimited command length, long line handling) added Apr 1999 by
  * Mike Coleman <mkc@acm.org>.
-*/
+ *
+ * Changes by Albert Cahalan, 2002-2003.
+ */
+
+#define VERSION "0.2.0"
 
 #include <ctype.h>
 #include <getopt.h>
@@ -19,37 +22,39 @@
 #include <sys/ioctl.h>
 #include <time.h>
 #include <unistd.h>
-
-#include "proc/version.h"
+#include <locale.h>
+#include "proc/procps.h"
 
 static struct option longopts[] = {
 	{"differences", optional_argument, 0, 'd'},
 	{"help", no_argument, 0, 'h'},
 	{"interval", required_argument, 0, 'n'},
+	{"no-title", no_argument, 0, 't'},
 	{"version", no_argument, 0, 'v'},
 	{0, 0, 0, 0}
 };
 
+static char usage[] =
+    "Usage: %s [-dhntv] [--differences[=cumulative]] [--help] [--interval=<n>] [--no-title] [--version] <command>\n";
+
+static char *progname;
+
 static int curses_started = 0;
 static int height = 24, width = 80;
 static int screen_size_changed = 0;
 static int first_screen = 1;
+static int show_title = 2;  // number of lines used, 2 or 0
 
-#define max(x,y) ((x) > (y) ? (x) : (y))
+#define min(x,y) ((x) > (y) ? (y) : (x))
 
-static void usage(const char *cmd)
+static void do_usage(void) NORETURN;
+static void do_usage(void)
 {
-	fprintf(stderr, "usage: %s [options] command\n", cmd);
-	fprintf(stderr, "    --differences[=cumulative], -d    "
-			"highlight changes between updates\n");
-	fprintf(stderr, "    --interval=<seconds>, -n          "
-			"seconds to wait between updates\n");
-	fprintf(stderr, "    --help                            "
-			"print a summary of the options\n");
-	fprintf(stderr, "    --version, -V                     "
-			"print version information and quit\n");
+	fprintf(stderr, usage, progname);
+	exit(1);
 }
 
+static void do_exit(int status) NORETURN;
 static void do_exit(int status)
 {
 	if (curses_started)
@@ -57,25 +62,23 @@ static void do_exit(int status)
 	exit(status);
 }
 
-/*
- * die - signal handler for SIGINT, SIGTERM, and SIGUP
- */
-static void die(int unused)
+/* signal handler */
+static void die(int notused) NORETURN;
+static void die(int notused)
 {
-	(void) unused;
+	(void) notused;
 	do_exit(0);
 }
 
-/*
- * winch_handler - signal handler for SIGWINCH
- */
-static void winch_handler(int unused)
+static void
+winch_handler(int notused)
 {
-	(void) unused;
+	(void) notused;
 	screen_size_changed = 1;
 }
 
-static void get_terminal_size(void)
+static void
+get_terminal_size(void)
 {
 	struct winsize w;
 	if (ioctl(2, TIOCGWINSZ, &w) == 0) {
@@ -86,60 +89,84 @@ static void get_terminal_size(void)
 	}
 }
 
-int main(int argc, char *argv[])
+int
+main(int argc, char *argv[])
 {
-	int opt;
-	int option_differences = 0, option_differences_cumulative = 0;
-	int interval = 2;
+	int optc;
+	int option_differences = 0,
+	    option_differences_cumulative = 0,
+	    option_help = 0, option_version = 0;
+	float interval = 2;
 	char *command;
 	int command_length = 0;	/* not including final \0 */
 
-	while ((opt = getopt_long(argc, argv, "d::n:hV", longopts,
-			NULL)) != EOF) {
-		switch (opt) {
+	setlocale(LC_ALL, "");
+	progname = argv[0];
+
+	while ((optc = getopt_long(argc, argv, "+d::hn:vt", longopts, (int *) 0))
+	       != EOF) {
+		switch (optc) {
 		case 'd':
 			option_differences = 1;
 			if (optarg)
 				option_differences_cumulative = 1;
 			break;
+		case 'h':
+			option_help = 1;
+			break;
+		case 't':
+			show_title = 0;
+			break;
 		case 'n':
 			{
-				char *s;
-				interval = strtol(optarg, &s, 10);
-				if (!*optarg || *s) {
-					usage(argv[0]);
-					return 1;
-				}
+				char *str;
+				interval = strtof(optarg, &str);
+				if (!*optarg || *str)
+					do_usage();
+				if(interval < 0.1)
+					interval = 0.1;
 			}
 			break;
-		case 'V':
-			display_version();
-			return 0;
-		case 'h':
-			usage(argv[0]);
-			return 0;
+		case 'v':
+			option_version = 1;
+			break;
 		default:
-			usage(argv[0]);
-			return 1;
+			do_usage();
+			break;
 		}
 	}
 
-	if (optind >= argc) {
-		usage(argv[0]);
-		return 1;
+	if (option_version) {
+		fprintf(stderr, "%s\n", VERSION);
+		if (!option_help)
+			exit(0);
 	}
 
+	if (option_help) {
+		fprintf(stderr, usage, progname);
+		fputs("  -d, --differences[=cumulative]\thighlight changes between updates\n", stderr);
+		fputs("\t\t(cumulative means highlighting is cumulative)\n", stderr);
+		fputs("  -h, --help\t\t\t\tprint a summary of the options\n", stderr);
+		fputs("  -n, --interval=<seconds>\t\tseconds to wait between updates\n", stderr);
+		fputs("  -v, --version\t\t\t\tprint the version number\n", stderr);
+		fputs("  -t, --no-title\t\t\tturns off showing the header\n", stderr);
+		exit(0);
+	}
+
+	if (optind >= argc)
+		do_usage();
+
 	command = strdup(argv[optind++]);
 	command_length = strlen(command);
 	for (; optind < argc; optind++) {
-		int s = strlen(argv[optind]);
 		char *endp;
-
-		command = realloc(command, command_length + s + 2);
-		endp = &command[command_length];
+		int s = strlen(argv[optind]);
+		command = realloc(command, command_length + s + 2);	/* space and \0 */
+		endp = command + command_length;
 		*endp = ' ';
-		command_length += s + 1;
-		strcpy(endp + 1, argv[optind]);
+		memcpy(endp + 1, argv[optind], s);
+		command_length += 1 + s;	/* space then string length */
+		command[command_length] = '\0';
 	}
 
 	get_terminal_size();
@@ -157,14 +184,14 @@ int main(int argc, char *argv[])
 	noecho();
 	cbreak();
 
-	while (1) {
+	for (;;) {
 		time_t t = time(NULL);
 		char *ts = ctime(&t);
 		int tsl = strlen(ts);
 		char *header;
 		FILE *p;
 		int x, y;
-		int last_was_eol;
+		int oldeolseen = 1;
 
 		if (screen_size_changed) {
 			get_terminal_size();
@@ -175,57 +202,44 @@ int main(int argc, char *argv[])
 			first_screen = 1;
 		}
 
-		/*
-		 * left justify interval and command, right justify time,
-		 * clipping all to fit window width
-		 */
-		asprintf(&header, "Every %ds: %.*s",
-			 interval, max(width - 1, command_length), command);
-		mvaddstr(0, 0, header);
-		if (strlen(header) > (unsigned int) (width - tsl - 1))
-			mvaddstr(0, width - tsl - 4, "...  ");
-		mvaddstr(0, width - tsl + 1, ts);
-		free(header);
+		if (show_title) {
+			// left justify interval and command,
+			// right justify time, clipping all to fit window width
+			asprintf(&header, "Every %.1fs: %.*s",
+				interval, min(width - 1, command_length), command);
+			mvaddstr(0, 0, header);
+			if (strlen(header) > (size_t) (width - tsl - 1))
+				mvaddstr(0, width - tsl - 4, "...  ");
+			mvaddstr(0, width - tsl + 1, ts);
+			free(header);
+		}
 
 		if (!(p = popen(command, "r"))) {
 			perror("popen");
 			do_exit(2);
 		}
 
-		last_was_eol = 0;
-		for (y = 2; y < height; y++) {
+		for (y = show_title; y < height; y++) {
 			int eolseen = 0, tabpending = 0;
 			for (x = 0; x < width; x++) {
 				int c = ' ';
 				int attr = 0;
 
 				if (!eolseen) {
-					/*
-					 * if there is a tab pending, just spit
-					 * spaces until the next stop instead
-					 * of reading characters
-					 */
-				      restart:
+					/* if there is a tab pending, just spit spaces until the
+					   next stop instead of reading characters */
 					if (!tabpending)
 						do
 							c = getc(p);
 						while (c != EOF && !isprint(c)
 						       && c != '\n'
 						       && c != '\t');
-
-					/* Avoid putting the newline first at the row if the last
-					 * character on the last row was not a newline.
-					 * This avoids empty lines when a newline gets pushed one char
-					 * off the end of a row.
-					 */
-					if (x == 0 && c == '\n'
-					    && !last_was_eol) {
-						last_was_eol = 1;
-						goto restart;
-					}
-
 					if (c == '\n')
-						eolseen = 1;
+						if (!oldeolseen && x == 0) {
+							x = -1;
+							continue;
+						} else
+							eolseen = 1;
 					else if (c == '\t')
 						tabpending = 1;
 					if (c == EOF || c == '\n' || c == '\t')
@@ -248,15 +262,15 @@ int main(int argc, char *argv[])
 				addch(c);
 				if (attr)
 					standend();
-				last_was_eol = eolseen;
 			}
+			oldeolseen = eolseen;
 		}
 
 		pclose(p);
 
 		first_screen = 0;
 		refresh();
-		sleep(interval);
+		usleep(interval * 1000000);
 	}
 
 	endwin();
